#!/usr/bin/perl
# rssedit.pl
# by Jonathan Eisenzopf. v1.0
# Copyright (c) 1999 Jonathan Eisenzopf. All Rights Reserved.
# See http://www.perlxml.com for more information
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# INCLUDES
use strict;
use CGI qw/:standard *table/;
use LWP::Simple;
use CGI::Carp;
use XML::RSS;

#CONSTANTS
my $rss_file = "/tmp/xml.rss";
my $output = 'save';

my %languages = (
    'af'    => 'Afrikaans',
    'sq'    => 'Albanian',
    'eu'    => 'Basque',
    'be'    => 'Belarusian',
    'bg'    => 'Bulgarian',
    'ca'    => 'Catalan',
    'zh-cn' => 'Chinese (Simplified)',
    'zh-tw' => 'Chinese (Traditional)',
    'hr'    => 'Croatian',
    'cs'    => 'Czech',
    'da'    => 'Danish',
    'nl'    => 'Dutch',
    'nl-be' => 'Dutch (Belgium)',
    'nl-nl' => 'Dutch (Netherlands)',
    'en'    => 'English',
    'en-au' => 'English (Australia)',
    'en-bz' => 'English (Belize)',
    'en-ca' => 'English (Canada)',
    'en-ie' => 'English (Ireland)',
    'en-jm' => 'English (Jamaica)',
    'en-nz' => 'English (New Zealand)',
    'en-ph' => 'English (Phillipines)',
    'en-za' => 'English (South Africa)',
    'en-tt' => 'English (Trinidad)',
    'en-gb' => 'English (United Kingdom)',
    'en-us' => 'English (United States)',
    'en-zw' => 'English (Zimbabwe)',
    'fo'    => 'Faeroese',
    'fi'    => 'Finnish',
    'fr'    => 'French',
    'fr-be' => 'French (Belgium)',
    'fr-ca' => 'French (Canada)',
    'fr-fr' => 'French (France)',
    'fr-lu' => 'French (Luxembourg)',
    'fr-mc' => 'French (Monaco)',
    'fr-ch' => 'French (Switzerland)',
    'gl'    => 'Galician',
    'gd'    => 'Gaelic',
    'de'    => 'German',
    'de-at' => 'German (Austria)',
    'de-de' => 'German (Germany)',
    'de-li' => 'German (Liechtenstein)',
    'de-lu' => 'German (Luxembourg)',
    'el'    => 'Greek',
    'hu'    => 'Hungarian',
    'is'    => 'Icelandic',
    'in'    => 'Indonesian',
    'ga'    => 'Irish',
    'it'    => 'Italian',
    'it-it' => 'Italian (Italy)',
    'it-ch' => 'Italian (Switzerland)',
    'ja'    => 'Japanese',
    'ko'    => 'Korean',
    'mk'    => 'Macedonian',
    'no'    => 'Norwegian',
    'pl'    => 'Polish',
    'pt'    => 'Portuguese',
    'pt-br' => 'Portuguese (Brazil)',
    'pt-pt' => 'Portuguese (Portugal)',
    'ro'    => 'Romanian',
    'ro-mo' => 'Romanian (Moldova)',
    'ro-ro' => 'Romanian (Romania)',
    'ru'    => 'Russian',
    'ru-mo' => 'Russian (Moldova)',
    'ru-ru' => 'Russian (Russia)',
    'sr'    => 'Serbian',
    'sk'    => 'Slovak',
    'sl'    => 'Slovenian',
    'es'    => 'Spanish',
    'es-ar' => 'Spanish (Argentina)',
    'es-bo' => 'Spanish (Bolivia)',
    'es-cl' => 'Spanish (Chile)',
    'es-co' => 'Spanish (Colombia)',
    'es-cr' => 'Spanish (Costa Rica)',
    'es-do' => 'Spanish (Dominican Republic)',
    'es-ec' => 'Spanish (Ecuador)',
    'es-sv' => 'Spanish (El Salvador)',
    'es-gt' => 'Spanish (Guatemala)',
    'es-hn' => 'Spanish (Honduras)',
    'es-mx' => 'Spanish (Mexico)',
    'es-ni' => 'Spanish (Nicaragua)',
    'es-pa' => 'Spanish (Panama)',
    'es-py' => 'Spanish (Paraguay)',
    'es-pe' => 'Spanish (Peru)',
    'es-pr' => 'Spanish (Puerto Rico)',
    'es-es' => 'Spanish (Spain)',
    'es-uy' => 'Spanish (Uruguay)',
    'es-ve' => 'Spanish (Venezuela)',
    'sv'    => 'Swedish',
    'sv-fi' => 'Swedish (Finland)',
    'sv-se' => 'Swedish (Sweden)',
    'tr'    => 'Turkish',
    'uk'    => 'Ukranian'
                 );

# MAIN
my $rss = new XML::Parser;
if (param('ACTION') eq 'Fetch') {
    print header;
    &fetch_rss(param('rss_url')) if param('rss_url');
    &print_form;
} elsif (param('ACTION') eq 'Build RSS') {
    &generate_RSS;
} else {
    print header;
    &print_form;
}


# SUBROUTINES
sub fetch_rss {
    my $url = shift;
    my $data = get($url);
    return "Error fetching $url" unless $data;
    my $rss = new XML::RSS;
    eval { $rss->parse($data) };
    return "Error parsing RSS file: $@" if $@;

    # set channel params
    param(-name=>'ctitle',  -value=>$rss->channel('title'));
    param(-name=>'cdesc',   -value=>$rss->channel('description'));
    param(-name=>'clink',   -value=>$rss->channel('link'));
    if ($rss->channel('language') =~ /\S+/) {
        param(-name=>'clang', -value=>$rss->channel('language'));
    } else {
        param(-name=>'clang', -value=>'en-us');
    }
    param(-name=>'crating', -value=>$rss->channel('rating'));
    param(-name=>'ceditor', -value=>$rss->channel('managingEditor'));
    param(-name=>'cwebmaster', -value=>$rss->channel('webMaster'));
    param(-name=>'ccopyright', -value=>$rss->channel('copyright'));
    param(-name=>'cpubdate', -value=>$rss->channel('pubDate'));
    param(-name=>'clastbuild', -value=>$rss->channel('lastBuildDate'));

    # set image params
    param(-name=>'ititle',  -value=>$rss->image('title'));
    param(-name=>'iurl',    -value=>$rss->image('url'));
    param(-name=>'ilink',   -value=>$rss->image('link'));
    param(-name=>'iwidth',  -value=>$rss->image('width'));
    param(-name=>'iheight', -value=>$rss->image('height'));
    param(-name=>'idesc',   -value=>$rss->image('description'));

    # set text input params
    param(-name=>'tititle', -value=>$rss->textinput('title'));
    param(-name=>'tidesc',  -value=>$rss->textinput('description'));
    param(-name=>'tiname',  -value=>$rss->textinput('name'));
    param(-name=>'tilink',  -value=>$rss->textinput('link'));

    # set item params
    for (my $i=0; $i < @{$rss->{'items'}}; $i++) {
        my $item = $rss->{'items'}->[$i];
        param(-name=>"item".($i+1)."_title", -value=>$item->{'title'});
        param(-name=>"item".($i+1)."_link", -value=>$item->{'link'});
	param(-name=>"item".($i+1)."_desc", -value=>$item->{'description'});
    }
}

sub generate_RSS {
    # create instance of XML::RSS
    my $rss = new XML::RSS (version => '0.91');

    # channel
    $rss->channel(
                  'title'       => param('ctitle'),
                  'link'        => param('clink'),
                  'description' => param('cdesc'),
                  'language'    => param('clang')
                  );
    $rss->channel('rating'      => param('crating')) if param('crating');
    $rss->channel('managingEditor' => param('ceditor')) if param('ceditor');
    $rss->channel('webMaster'   => param('cwebmaster')) if param('cwebmaster');
    $rss->channel('copyright'   => param('ccopyright')) if param('ccopyright');
    $rss->channel('pubDate'     => param('cpubdate')) if param('cpubdate');
    $rss->channel('lastBuildDate' => param('clastbuild')) if param('clastbuild');

    # image
    if (param('iurl') =~ /\S+/) {
        $rss->image(
                    'title' => param('ititle'),
                    'url'   => param('iurl'),
                    'link'  => param('ilink')
                    );
	$rss->image('description' => param('idesc')) if param('idesc');
    }

    # textinput
    if (param('tilink')) {
        $rss->textinput(
                        'title'       => param('tititle'),
                        'description' => param('tidesc'),
                        'name'        => param('tiname'),
                        'link'        => param('tilink')
                        );
    }

    # items
    for (my $i=1; $i <= 15; $i++) {
        $rss->add_item(
                       'title' => param("item".$i."_title"),
                       'link'  => param("item".$i."_link"),
		       'description'  => param("item".$i."_desc")
                       );
    }

    # new item 1
    if (param('newtitle1') && param('newlink1') && param('newdesc1')) {
	pop(@{$rss->{'items'}}) if (@{$rss->{'items'}} >= 15);
	$rss->add_item(
		       'title' => param('newtitle1'),
		       'link'  => param('newlink1'),
		       'description' => param('newdesc1'),
		       'mode'  => 'insert'
		       );
    }

    # new item 2
    if (param('newtitle2') && param('newlink2') && param('newdesc2')) {
	pop(@{$rss->{'items'}}) if (@{$rss->{'items'}} >= 15);
	$rss->add_item(
		       'title' => param('newtitle2'),
		       'link'  => param('newlink2'),
		       'description' => param('newdesc2'),
		       'mode'  => 'insert'
		       );
    }

    if ($output eq 'save') {
	$rss->save($rss_file);
	print header;
	&print_form;
    } else {
	# print RSS
	print header('text/xml');
	print $rss->as_string;
    }
}

sub print_form {
    my $ctitle     = param('ctitle');
    my $cdesc      = param('cdesc');
    my $clink      = param('clink');
    my $clang      = param('clang') ? param('clang') : 'en-us';
    my $crating    = param('crating');
    my $ceditor    = param('ceditor');
    my $cwebmaster = param('cwebmaster');
    my $ccopyright = param('ccopyright');
    my $cpubdate   = param('cpubdate');
    my $clastbuild = param('clastbuild');
    my $ititle     = param('ititle');
    my $iurl       = param('iurl');
    my $ilink      = param('ilink');
    my $iwidth     = param('iwidth');
    my $iheight    = param('iheight');
    my $idesc      = param('idesc');
    my $tititle    = param('tititle');
    my $tidesc     = param('tidesc');
    my $tiname     = param('tiname');
    my $tilink     = param('tilink');
    my $rss_url    = param('rss_url');

    print start_html(-title=>'RSS Channel Editor',
               -BGCOLOR=>'#FFFFFF');

    print h1({-align=>'CENTER'},'RSS Channel Editor');

    print startform(-method=>'POST',
                    -action=>url());

    print table({-border=>undef, -align=>'center'});

    # RSS URL
    print Tr({-align=>'LEFT',-valign=>'MIDDLE'},
             [
              th({-colspan=>2,-BGCOLOR=>'#000000'},
                 font({-color=>'#FFFFFF',-font=>'Arial,Helvetica',-size=>'+1'},
                      'RSS URL)')
                 ),
              td({-align=>'right'},[b('RSS URL(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#rss_url">?</a>):'),
                  textfield(-name=>'rss_url',
                             -default=>$rss_url,
                             -size=>50
                            ),
                  submit(-name=>'ACTION',-value=>'Fetch')
                ])
              ]);


    # new channel items
    #1

    print Tr({-align=>'LEFT',-valign=>'MIDDLE'},
	     [
	      th({-colspan=>2,-BGCOLOR=>'#000000'}, 
		 font({-color=>'#FFFFFF',-font=>'Arial,Helvetica',-size=>'+1'},"New Item 1")
		 ),
	      
	          td({-align=>'right'},[b('Title(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_title">?</a>):'),
                     textfield(-name=>'newtitle1',
                               -size=>50,
			       -default=>''
                               ),
                     submit(-name=>'ACTION',-value=>'Build RSS')
                     ]),

                  td({-align=>'right'},[b('Link(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_link">?</a>):'),
                     textfield(-name=>'newlink1',
                               -size=>50,
			       -default=>''
                               ),

                     ]),

		  td({-align=>'right'},[b('Desc(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_desc">?</a>):'),
                     textarea(-name=>'newdesc1',			      
			      -columns=>50,
			      -rows=>5,
			      -default=>''
                               ),

                     ])
              ]);

    # new channel items
    # 2
    print Tr({-align=>'LEFT',-valign=>'MIDDLE'},
	     [
	      th({-colspan=>2,-BGCOLOR=>'#000000'}, 
		 font({-color=>'#FFFFFF',-font=>'Arial,Helvetica',-size=>'+1'},"New Item 2")
		 ),
	      
	          td({-align=>'right'},[b('Title(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_title">?</a>):'),
                     textfield(-name=>'newtitle2',
                               -size=>50,
			       -default=>''
                               ),
                     submit(-name=>'ACTION',-value=>'Build RSS')
                     ]),

                  td({-align=>'right'},[b('Link(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_link">?</a>):'),
                     textfield(-name=>'newlink2',
                               -size=>50,
			       -default=>''
                               ),

                     ]),

		  td({-align=>'right'},[b('Desc(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_desc">?</a>):'),
                     textarea(-name=>'newdesc2',
			      -columns=>50,
			      -rows=>5,
			      -default=>''
			      
                               ),

                     ])
              ]);


    # channel summary
    print Tr({-align=>'LEFT',-valign=>'MIDDLE'},
             [
              th({-colspan=>2,-BGCOLOR=>'#000000'},
                 font({-color=>'#FFFFFF',-font=>'Arial,Helvetica',-size=>'+1'},
                      'Channel Summary')
                 ),
              td({-align=>'right'},[b('Channel Title(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_title">?</a>):'),
                  textfield(-name=>'ctitle',
                            -default=>$ctitle,
                            -size=>50
                            )
                  ]),

              td({-align=>'right'},[b('Channel Link(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_link">?</a>):'),
                  textfield(-name=>'clink',
                           -default=>$clink,
                           -size=>50
                           )
                  ]),

              td({-align=>'right'},[b('Description(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_description">?</a>):'),
                  textarea(-name=>'cdesc',
                           -default=>$cdesc,
                           -rows=>5,
                           -columns=>50
                           )
                  ]),

              td({-align=>'right'},[b('Language(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#language">?</a>):'),
                 popup_menu(-name=>'clang',
                             -values=>[sort(keys(%languages))],
                             -default=>$clang,
                             -labels=>\%languages
                             )
                  ]),

	      td({-align=>'right'},[b('PICS Rating(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_rating">?</a>):'),
                  textfield(-name=>'crating',
                           -default=>$crating,
                           -size=>50
                           )
                  ]),

	      td({-align=>'right'},[b('Managing Editor(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_editor">?</a>):'),
                  textfield(-name=>'ceditor',
                           -default=>$ceditor,
                           -size=>50
                           )
                  ]),

	      td({-align=>'right'},[b('Webmaster(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_webmaster">?</a>):'),
                  textfield(-name=>'cwebmaster',
                           -default=>$cwebmaster,
                           -size=>50
                           )
                  ]),


	      td({-align=>'right'},[b('Copyright(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_copyright">?</a>):'),
                  textfield(-name=>'ccopyright',
                           -default=>$ccopyright,
                           -size=>50
                           )
                  ]),

	      td({-align=>'right'},[b('Publish Date(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_pubdate">?</a>):'),
                  textfield(-name=>'cpubdate',
                           -default=>$cpubdate,
                           -size=>50
                           )
                  ]),

	      td({-align=>'right'},[b('Last Build(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_lastbuild">?</a>):'),
                  textfield(-name=>'clastbuild',
                           -default=>$clastbuild,
                           -size=>50
                           )
                  ])
              ]);

    # channel image
    print Tr({-align=>'LEFT',-valign=>'MIDDLE'},
             [
              th({-colspan=>2,-BGCOLOR=>'#000000'},
                 font({-color=>'#FFFFFF',-font=>'Arial,Helvetica',-size=>'+1'},
                      'Channel Image')
                 ),

              td({-align=>'right'},[b('Image Title(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#image_title">?</a>):'),
                  textfield(-name=>'ititle',
                           -default=>$ititle,
                           -size=>50
                           )
                  ]),

              td({-align=>'right'},[b('Image URL(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#image_url">?</a>):'),
                  textfield(-name=>'iurl',
                           -default=>$iurl,
                           -size=>50
                           )
                  ]),

              td({-align=>'right'},[b('Image Link(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#image_link">?</a>):'),
                  textfield(-name=>'ilink',
                           -default=>$ilink,
                           -size=>50
                           )
                  ]),

	      td({-align=>'right'},[b('Image Width(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#image_width">?</a>):'),
                  textfield(-name=>'iwidth',
                           -default=>$iwidth,
                           -size=>50
                           )
                  ]),

	      td({-align=>'right'},[b('Image Height(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#image_height">?</a>):'),
                  textfield(-name=>'iheight',
                           -default=>$iheight,
                           -size=>50
                           )
                  ]),

	      td({-align=>'right'},[b('Image Desc(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#image_desc">?</a>):'),
                  textarea(-name=>'idesc',
                           -default=>$idesc,
                           -columns=>50,
			   -rows=>5
                           )
                  ])
              ]);


    # channel items
    for (my $i=1; $i<=15; $i++) {
        my %title;
        my %link;
	my %desc;
        $title{'name'} = "item".$i."_title";
        $title{'value'} = param($title{'title'});
	$desc{'name'} = "item".$i."_desc";
	$desc{'value'} = param($desc{'name'});
        $link{'name'} = "item".$i."_link";
        $link{'value'} = param($link{'name'});

        print Tr({-align=>'LEFT',-valign=>'MIDDLE'},
                 [
                  th({-colspan=>2,-BGCOLOR=>'#000000'},

font({-color=>'#FFFFFF',-font=>'Arial,Helvetica',-size=>'+1'},"Item$i")
                     ),

                  td({-align=>'right'},[b('Title(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_title">?</a>):'),
                     textfield(-name=>$title{'name'},
                               -default=>$title{'value'},
                               -size=>50
                               ),
                     submit(-name=>'ACTION',-value=>'Build RSS')
                     ]),

                  td({-align=>'right'},[b('Link(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_link">?</a>):'),
                     textfield(-name=>$link{'name'},
                               -default=>$link{'value'},
                               -size=>50
                               ),

                     ]),

		  td({-align=>'right'},[b('Desc(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_desc">?</a>):'),
                     textarea(-name=>$desc{'name'},
			      -default=>$desc{'value'},
			      -columns=>50,
			      -rows=>5
			      
                               ),
                     ])
              ]);
    }

    # channel textarea
    print Tr({-align=>'LEFT',-valign=>'MIDDLE'},
             [
              th({-colspan=>2,-BGCOLOR=>'#000000'},
                 font({-color=>'#FFFFFF',-font=>'Arial,Helvetica',-size=>'+1'},
                      'Text Input')
                 ),

              td({-align=>'right'},[b('Input Title(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#input_title">?</a>):'),
                  textfield(-name=>'tititle',
                           -default=>$tititle,
                           -size=>50
                           )
                  ]),

              td({-align=>'right'},[b('Input Name(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#input_name">?</a>):'),
                  textfield(-name=>'tiname',
                           -default=>$tiname,
                           -size=>50
                           )
                  ]),

              td({-align=>'right'},[b('Input Link(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#input_link">?</a>):'),
                  textfield(-name=>'tilink',
                           -default=>$tilink,
                           -size=>50
                           )
                  ]),

              td({-align=>'right'},[b('Input Desc(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#input_description">?</a>):'),
                  textarea(-name=>'tidesc',
                           -default=>$tidesc,
                           -columns=>50,
                           -rows=>5
                           )
                  ])
              ]);


    print end_table;
    print end_form;
}
